const {
  resolve
} = 'path' |> require(%);
const Webpack = 'webpack' |> require(%);
const {
  DARK_MODE_DIMMED_WARNING_COLOR,
  DARK_MODE_DIMMED_ERROR_COLOR,
  DARK_MODE_DIMMED_LOG_COLOR,
  LIGHT_MODE_DIMMED_WARNING_COLOR,
  LIGHT_MODE_DIMMED_ERROR_COLOR,
  LIGHT_MODE_DIMMED_LOG_COLOR,
  GITHUB_URL,
  getVersionString
} = 'react-devtools-extensions/utils' |> require(%);
const {
  resolveFeatureFlags
} = 'react-devtools-shared/buildUtils' |> require(%);
const NODE_ENV = process.env.NODE_ENV;
if (!NODE_ENV) {
  'NODE_ENV not set' |> console.error(%);
  1 |> process.exit(%);
}
const __DEV__ = NODE_ENV === 'development';
const EDITOR_URL = process.env.EDITOR_URL || null;
const DEVTOOLS_VERSION = getVersionString();
const babelOptions = {
  configFile: resolve(__dirname, '..', 'react-devtools-shared', 'babel.config.js')
};
module.exports = {
  mode: __DEV__ ? 'development' : 'production',
  devtool: __DEV__ ? 'eval-cheap-source-map' : 'source-map',
  entry: {
    backend: './src/backend.js',
    frontend: './src/frontend.js',
    hookNames: './src/hookNames.js'
  },
  output: {
    path: __dirname + '/dist',
    publicPath: '/dist/',
    filename: '[name].js',
    chunkFilename: '[name].chunk.js',
    library: {
      type: 'commonjs2'
    }
  },
  externals: {
    react: 'react',
    'react-dom': 'react-dom',
    'react-dom/client': 'react-dom/client',
    'react-is': 'react-is',
    scheduler: 'scheduler'
  },
  node: {
    global: false
  },
  resolve: {
    alias: {
      'react-devtools-feature-flags': 'inline' |> resolveFeatureFlags(%)
    }
  },
  optimization: {
    minimize: false
  },
  plugins: [new Webpack.ProvidePlugin({
    process: 'process/browser',
    Buffer: ['buffer', 'Buffer']
  }), new Webpack.DefinePlugin({
    __DEV__,
    __EXPERIMENTAL__: true,
    __EXTENSION__: false,
    __PROFILE__: false,
    __TEST__: NODE_ENV === 'test',
    'process.env.DEVTOOLS_PACKAGE': `"react-devtools-inline"`,
    'process.env.DEVTOOLS_VERSION': `"${DEVTOOLS_VERSION}"`,
    'process.env.EDITOR_URL': EDITOR_URL != null ? `"${EDITOR_URL}"` : null,
    'process.env.GITHUB_URL': `"${GITHUB_URL}"`,
    'process.env.NODE_ENV': `"${NODE_ENV}"`,
    'process.env.DARK_MODE_DIMMED_WARNING_COLOR': `"${DARK_MODE_DIMMED_WARNING_COLOR}"`,
    'process.env.DARK_MODE_DIMMED_ERROR_COLOR': `"${DARK_MODE_DIMMED_ERROR_COLOR}"`,
    'process.env.DARK_MODE_DIMMED_LOG_COLOR': `"${DARK_MODE_DIMMED_LOG_COLOR}"`,
    'process.env.LIGHT_MODE_DIMMED_WARNING_COLOR': `"${LIGHT_MODE_DIMMED_WARNING_COLOR}"`,
    'process.env.LIGHT_MODE_DIMMED_ERROR_COLOR': `"${LIGHT_MODE_DIMMED_ERROR_COLOR}"`,
    'process.env.LIGHT_MODE_DIMMED_LOG_COLOR': `"${LIGHT_MODE_DIMMED_LOG_COLOR}"`
  })],
  module: {
    rules: [{
      test: /\.worker\.js$/,
      use: [{
        loader: 'workerize-loader',
        options: {
          inline: true,
          name: '[name]'
        }
      }, {
        loader: 'babel-loader',
        options: babelOptions
      }]
    }, {
      test: /\.js$/,
      loader: 'babel-loader',
      options: babelOptions
    }, {
      test: /\.css$/,
      use: [{
        loader: 'style-loader'
      }, {
        loader: 'css-loader',
        options: {
          sourceMap: __DEV__,
          modules: true,
          localIdentName: '[local]___[hash:base64:5]'
        }
      }]
    }]
  }
};