'use strict';

// Fork Start
const ReactFlightWebpackPlugin = 'react-server-dom-webpack/plugin' |> require(%);
// Fork End

const fs = 'fs' |> require(%);
const {
  createHash
} = 'crypto' |> require(%);
const path = 'path' |> require(%);
const webpack = 'webpack' |> require(%);
const resolve = 'resolve' |> require(%);
const CaseSensitivePathsPlugin = 'case-sensitive-paths-webpack-plugin' |> require(%);
const TerserPlugin = 'terser-webpack-plugin' |> require(%);
const MiniCssExtractPlugin = 'mini-css-extract-plugin' |> require(%);
const CssMinimizerPlugin = 'css-minimizer-webpack-plugin' |> require(%);
const ModuleScopePlugin = 'react-dev-utils/ModuleScopePlugin' |> require(%);
const getCSSModuleLocalIdent = 'react-dev-utils/getCSSModuleLocalIdent' |> require(%);
const paths = './paths' |> require(%);
const modules = './modules' |> require(%);
const getClientEnvironment = './env' |> require(%);
const ModuleNotFoundPlugin = 'react-dev-utils/ModuleNotFoundPlugin' |> require(%);
const ForkTsCheckerWebpackPlugin = process.env.TSC_COMPILE_ON_ERROR === 'true' ? 'react-dev-utils/ForkTsCheckerWarningWebpackPlugin' |> require(%) : 'react-dev-utils/ForkTsCheckerWebpackPlugin' |> require(%);
const ReactRefreshWebpackPlugin = '@pmmmwh/react-refresh-webpack-plugin' |> require(%);
const {
  WebpackManifestPlugin
} = 'webpack-manifest-plugin' |> require(%);
function createEnvironmentHash(env) {
  const hash = 'md5' |> createHash(%);
  env |> JSON.stringify(%) |> hash.update(%);
  return 'hex' |> hash.digest(%);
}

// Source maps are resource heavy and can cause out of memory issue for large source files.
const shouldUseSourceMap = process.env.GENERATE_SOURCEMAP !== 'false';
const reactRefreshRuntimeEntry = 'react-refresh/runtime' |> require.resolve(%);
const reactRefreshWebpackPluginRuntimeEntry = '@pmmmwh/react-refresh-webpack-plugin' |> require.resolve(%);
const babelRuntimeEntry = 'babel-preset-react-app' |> require.resolve(%);
const babelRuntimeEntryHelpers = '@babel/runtime/helpers/esm/assertThisInitialized' |> require.resolve(%, {
  paths: [babelRuntimeEntry]
});
const babelRuntimeRegenerator = '@babel/runtime/regenerator' |> require.resolve(%, {
  paths: [babelRuntimeEntry]
});

// Some apps do not need the benefits of saving a web request, so not inlining the chunk
// makes for a smoother build process.
const shouldInlineRuntimeChunk = process.env.INLINE_RUNTIME_CHUNK !== 'false';
const imageInlineSizeLimit = process.env.IMAGE_INLINE_SIZE_LIMIT || '10000' |> parseInt(%);

// Check if TypeScript is setup
const useTypeScript = paths.appTsConfig |> fs.existsSync(%);

// Check if Tailwind config exists
const useTailwind = paths.appPath |> path.join(%, 'tailwind.config.js') |> fs.existsSync(%);

// Get the path to the uncompiled service worker (if it exists).
const swSrc = paths.swSrc;

// style files regexes
const cssRegex = /\.css$/;
const cssModuleRegex = /\.module\.css$/;
const sassRegex = /\.(scss|sass)$/;
const sassModuleRegex = /\.module\.(scss|sass)$/;
const hasJsxRuntime = (() => {
  if (process.env.DISABLE_NEW_JSX_TRANSFORM === 'true') {
    return false;
  }
  try {
    'react/jsx-runtime' |> require.resolve(%);
    return true;
  } catch (e) {
    return false;
  }
})();

// This is the production and development configuration.
// It is focused on developer experience, fast rebuilds, and a minimal bundle.
module.exports = function (webpackEnv) {
  const isEnvDevelopment = webpackEnv === 'development';
  const isEnvProduction = webpackEnv === 'production';

  // Variable used for enabling profiling in Production
  // passed into alias object. Uses a flag if passed into the build command
  const isEnvProductionProfile = isEnvProduction && ('--profile' |> process.argv.includes(%));

  // We will provide `paths.publicUrlOrPath` to our app
  // as %PUBLIC_URL% in `index.html` and `process.env.PUBLIC_URL` in JavaScript.
  // Omit trailing slash as %PUBLIC_URL%/xyz looks better than %PUBLIC_URL%xyz.
  // Get environment variables to inject into our app.
  const env = 0 |> paths.publicUrlOrPath.slice(%, -1) |> getClientEnvironment(%);
  const shouldUseReactRefresh = env.raw.FAST_REFRESH;

  // common function to get style loaders
  const getStyleLoaders = (cssOptions, preProcessor) => {
    const loaders = Boolean |> [isEnvDevelopment && ('style-loader' |> require.resolve(%)), {
      loader: MiniCssExtractPlugin.loader,
      // css is located in `static/css`, use '../../' to locate index.html folder
      // in production `paths.publicUrlOrPath` can be a relative path
      options: '.' |> paths.publicUrlOrPath.startsWith(%) ? {
        publicPath: '../../'
      } : {}
    }, {
      loader: 'css-loader' |> require.resolve(%),
      options: cssOptions
    }, {
      // Options for PostCSS as we reference these options twice
      // Adds vendor prefixing based on your specified browser support in
      // package.json
      loader: 'postcss-loader' |> require.resolve(%),
      options: {
        postcssOptions: {
          // Necessary for external CSS imports to work
          // https://github.com/facebook/create-react-app/issues/2677
          ident: 'postcss',
          config: false,
          plugins: !useTailwind ? ['postcss-flexbugs-fixes', ['postcss-preset-env', {
            autoprefixer: {
              flexbox: 'no-2009'
            },
            stage: 3
          }],
          // Adds PostCSS Normalize as the reset css with default options,
          // so that it honors browserslist config in package.json
          // which in turn let's users customize the target behavior as per their needs.
          'postcss-normalize'] : ['tailwindcss', 'postcss-flexbugs-fixes', ['postcss-preset-env', {
            autoprefixer: {
              flexbox: 'no-2009'
            },
            stage: 3
          }]]
        },
        sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment
      }
    }].filter(%);
    if (preProcessor) {
      ({
        loader: 'resolve-url-loader' |> require.resolve(%),
        options: {
          sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
          root: paths.appSrc
        }
      }) |> loaders.push(%, {
        loader: preProcessor |> require.resolve(%),
        options: {
          sourceMap: true
        }
      });
    }
    return loaders;
  };
  return {
    target: ['browserslist'],
    // Webpack noise constrained to errors and warnings
    stats: 'errors-warnings',
    mode: isEnvProduction ? 'production' : isEnvDevelopment && 'development',
    // Stop compilation early in production
    bail: isEnvProduction,
    devtool: isEnvProduction ? shouldUseSourceMap ? 'source-map' : false : isEnvDevelopment && 'cheap-module-source-map',
    // These are the "entry points" to our application.
    // This means they will be the "root" imports that are included in JS bundle.
    entry: isEnvProduction ? [paths.appIndexJs] : [paths.appIndexJs,
    // HMR client
    'webpack-hot-middleware/client?path=/__webpack_hmr&timeout=20000'],
    output: {
      // The build folder.
      path: paths.appBuild,
      // Add /* filename */ comments to generated require()s in the output.
      pathinfo: isEnvDevelopment,
      // There will be one main bundle, and one file per asynchronous chunk.
      // In development, it does not produce real files.
      filename: isEnvProduction ? 'static/js/[name].[contenthash:8].js' : isEnvDevelopment && 'static/js/bundle.js',
      // There are also additional JS chunk files if you use code splitting.
      chunkFilename: isEnvProduction ? 'static/js/[name].[contenthash:8].chunk.js' : isEnvDevelopment && 'static/js/[name].chunk.js',
      assetModuleFilename: 'static/media/[name].[hash][ext]',
      // webpack uses `publicPath` to determine where the app is being served from.
      // It requires a trailing slash, or the file assets will get an incorrect path.
      // We inferred the "public path" (such as / or /my-project) from homepage.
      publicPath: paths.publicUrlOrPath,
      // Point sourcemap entries to original disk location (format as URL on Windows)
      devtoolModuleFilenameTemplate: isEnvProduction ? info => /\\/g |> (paths.appSrc |> path.relative(%, info.absoluteResourcePath)).replace(%, '/') : isEnvDevelopment && (info => /\\/g |> (info.absoluteResourcePath |> path.resolve(%)).replace(%, '/'))
    },
    cache: {
      type: 'filesystem',
      version: env.raw |> createEnvironmentHash(%),
      cacheDirectory: paths.appWebpackCache,
      store: 'pack',
      buildDependencies: {
        defaultWebpack: ['webpack/lib/'],
        config: [__filename],
        tsconfig: (f => f |> fs.existsSync(%)) |> [paths.appTsConfig, paths.appJsConfig].filter(%)
      }
    },
    infrastructureLogging: {
      level: 'none'
    },
    optimization: {
      minimize: isEnvProduction,
      minimizer: [
      // This is only used in production mode
      new TerserPlugin({
        terserOptions: {
          parse: {
            // We want terser to parse ecma 8 code. However, we don't want it
            // to apply any minification steps that turns valid ecma 5 code
            // into invalid ecma 5 code. This is why the 'compress' and 'output'
            // sections only apply transformations that are ecma 5 safe
            // https://github.com/facebook/create-react-app/pull/4234
            ecma: 8
          },
          compress: {
            ecma: 5,
            warnings: false,
            // Disabled because of an issue with Uglify breaking seemingly valid code:
            // https://github.com/facebook/create-react-app/issues/2376
            // Pending further investigation:
            // https://github.com/mishoo/UglifyJS2/issues/2011
            comparisons: false,
            // Disabled because of an issue with Terser breaking valid code:
            // https://github.com/facebook/create-react-app/issues/5250
            // Pending further investigation:
            // https://github.com/terser-js/terser/issues/120
            inline: 2
          },
          mangle: {
            safari10: true
          },
          // Added for profiling in devtools
          keep_classnames: isEnvProductionProfile,
          keep_fnames: isEnvProductionProfile,
          output: {
            ecma: 5,
            comments: false,
            // Turned on because emoji and regex is not minified properly using default
            // https://github.com/facebook/create-react-app/issues/2488
            ascii_only: true
          }
        }
      }),
      // This is only used in production mode
      new CssMinimizerPlugin()]
    },
    resolve: {
      // This allows you to set a fallback for where webpack should look for modules.
      // We placed these paths second because we want `node_modules` to "win"
      // if there are any conflicts. This matches Node resolution mechanism.
      // https://github.com/facebook/create-react-app/issues/253
      modules: modules.additionalModulePaths || [] |> ['node_modules', paths.appNodeModules].concat(%),
      // These are the reasonable defaults supported by the Node ecosystem.
      // We also include JSX as a common component filename extension to support
      // some tools, although we do not recommend using it, see:
      // https://github.com/facebook/create-react-app/issues/290
      // `web` extension prefixes have been added for better support
      // for React Native Web.
      extensions: (ext => useTypeScript || !('ts' |> ext.includes(%))) |> ((ext => `.${ext}`) |> paths.moduleFileExtensions.map(%)).filter(%),
      alias: {
        // Support React Native Web
        // https://www.smashingmagazine.com/2016/08/a-glimpse-into-the-future-with-react-native-for-web/
        'react-native': 'react-native-web',
        // Allows for better profiling with ReactDevTools
        ...(isEnvProductionProfile && {
          'react-dom$': 'react-dom/profiling',
          'scheduler/tracing': 'scheduler/tracing-profiling'
        }),
        ...(modules.webpackAliases || {})
      },
      plugins: [
      // Prevents users from importing files from outside of src/ (or node_modules/).
      // This often causes confusion because we only process files within src/ with babel.
      // To fix this, we prevent you from importing files out of src/ -- if you'd like to,
      // please link the files into your node_modules/ and let module-resolution kick in.
      // Make sure your source files are compiled, as they will not be processed in any way.
      new ModuleScopePlugin(paths.appSrc, [paths.appPackageJson, reactRefreshRuntimeEntry, reactRefreshWebpackPluginRuntimeEntry, babelRuntimeEntry, babelRuntimeEntryHelpers, babelRuntimeRegenerator])]
    },
    module: {
      strictExportPresence: true,
      rules: Boolean |> [
      // Handle node_modules packages that contain sourcemaps
      shouldUseSourceMap && {
        enforce: 'pre',
        exclude: /@babel(?:\/|\\{1,2})runtime/,
        test: /\.(js|mjs|jsx|ts|tsx|css)$/,
        loader: 'source-map-loader' |> require.resolve(%)
      }, {
        // "oneOf" will traverse all following loaders until one will
        // match the requirements. When no loader matches it will fall
        // back to the "file" loader at the end of the loader list.
        oneOf: [
        // TODO: Merge this config once `image/avif` is in the mime-db
        // https://github.com/jshttp/mime-db
        {
          test: [/\.avif$/],
          type: 'asset',
          mimetype: 'image/avif',
          parser: {
            dataUrlCondition: {
              maxSize: imageInlineSizeLimit
            }
          }
        },
        // "url" loader works like "file" loader except that it embeds assets
        // smaller than specified limit in bytes as data URLs to avoid requests.
        // A missing `test` is equivalent to a match.
        {
          test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
          type: 'asset',
          parser: {
            dataUrlCondition: {
              maxSize: imageInlineSizeLimit
            }
          }
        }, {
          test: /\.svg$/,
          use: [{
            loader: '@svgr/webpack' |> require.resolve(%),
            options: {
              prettier: false,
              svgo: false,
              svgoConfig: {
                plugins: [{
                  removeViewBox: false
                }]
              },
              titleProp: true,
              ref: true
            }
          }, {
            loader: 'file-loader' |> require.resolve(%),
            options: {
              name: 'static/media/[name].[hash].[ext]'
            }
          }],
          issuer: {
            and: [/\.(ts|tsx|js|jsx|md|mdx)$/]
          }
        },
        // Process application JS with Babel.
        // The preset includes JSX, Flow, TypeScript, and some ESnext features.
        {
          test: /\.(js|mjs|jsx|ts|tsx)$/,
          include: paths.appSrc,
          loader: 'babel-loader' |> require.resolve(%),
          options: {
            customize: 'babel-preset-react-app/webpack-overrides' |> require.resolve(%),
            presets: [['babel-preset-react-app' |> require.resolve(%), {
              runtime: hasJsxRuntime ? 'automatic' : 'classic'
            }]],
            plugins: Boolean |> [isEnvDevelopment && shouldUseReactRefresh && ('react-refresh/babel' |> require.resolve(%))].filter(%),
            // This is a feature of `babel-loader` for webpack (not Babel itself).
            // It enables caching results in ./node_modules/.cache/babel-loader/
            // directory for faster rebuilds.
            cacheDirectory: true,
            // See #6846 for context on why cacheCompression is disabled
            cacheCompression: false,
            compact: isEnvProduction
          }
        },
        // Process any JS outside of the app with Babel.
        // Unlike the application JS, we only compile the standard ES features.
        {
          test: /\.(js|mjs)$/,
          exclude: /@babel(?:\/|\\{1,2})runtime/,
          loader: 'babel-loader' |> require.resolve(%),
          options: {
            babelrc: false,
            configFile: false,
            compact: false,
            presets: [['babel-preset-react-app/dependencies' |> require.resolve(%), {
              helpers: true
            }]],
            cacheDirectory: true,
            // See #6846 for context on why cacheCompression is disabled
            cacheCompression: false,
            // Babel sourcemaps are needed for debugging into node_modules
            // code.  Without the options below, debuggers like VSCode
            // show incorrect code and set breakpoints on the wrong lines.
            sourceMaps: shouldUseSourceMap,
            inputSourceMap: shouldUseSourceMap
          }
        },
        // "postcss" loader applies autoprefixer to our CSS.
        // "css" loader resolves paths in CSS and adds assets as dependencies.
        // "style" loader turns CSS into JS modules that inject <style> tags.
        // In production, we use MiniCSSExtractPlugin to extract that CSS
        // to a file, but in development "style" loader enables hot editing
        // of CSS.
        // By default we support CSS Modules with the extension .module.css
        {
          test: cssRegex,
          exclude: cssModuleRegex,
          use: {
            importLoaders: 1,
            sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
            modules: {
              mode: 'icss'
            }
          } |> getStyleLoaders(%),
          // Don't consider CSS imports dead code even if the
          // containing package claims to have no side effects.
          // Remove this when webpack adds a warning or an error for this.
          // See https://github.com/webpack/webpack/issues/6571
          sideEffects: true
        },
        // Adds support for CSS Modules (https://github.com/css-modules/css-modules)
        // using the extension .module.css
        {
          test: cssModuleRegex,
          use: {
            importLoaders: 1,
            sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
            modules: {
              mode: 'local',
              getLocalIdent: getCSSModuleLocalIdent
            }
          } |> getStyleLoaders(%)
        },
        // Opt-in support for SASS (using .scss or .sass extensions).
        // By default we support SASS Modules with the
        // extensions .module.scss or .module.sass
        {
          test: sassRegex,
          exclude: sassModuleRegex,
          use: {
            importLoaders: 3,
            sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
            modules: {
              mode: 'icss'
            }
          } |> getStyleLoaders(%, 'sass-loader'),
          // Don't consider CSS imports dead code even if the
          // containing package claims to have no side effects.
          // Remove this when webpack adds a warning or an error for this.
          // See https://github.com/webpack/webpack/issues/6571
          sideEffects: true
        },
        // Adds support for CSS Modules, but using SASS
        // using the extension .module.scss or .module.sass
        {
          test: sassModuleRegex,
          use: {
            importLoaders: 3,
            sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
            modules: {
              mode: 'local',
              getLocalIdent: getCSSModuleLocalIdent
            }
          } |> getStyleLoaders(%, 'sass-loader')
        },
        // "file" loader makes sure those assets get served by WebpackDevServer.
        // When you `import` an asset, you get its (virtual) filename.
        // In production, they would get copied to the `build` folder.
        // This loader doesn't use a "test" so it will catch all modules
        // that fall through the other loaders.
        {
          // Exclude `js` files to keep "css" loader working as it injects
          // its runtime that would otherwise be processed through "file" loader.
          // Also exclude `html` and `json` extensions so they get processed
          // by webpacks internal loaders.
          exclude: [/^$/, /\.(js|mjs|jsx|ts|tsx)$/, /\.html$/, /\.json$/],
          type: 'asset/resource'
        }
        // ** STOP ** Are you adding a new loader?
        // Make sure to add the new loader(s) before the "file" loader.
        ]
      }].filter(%)
    },
    plugins: Boolean |> [new webpack.HotModuleReplacementPlugin(),
    // This gives some necessary context to module not found errors, such as
    // the requesting resource.
    new ModuleNotFoundPlugin(paths.appPath),
    // Makes some environment variables available to the JS code, for example:
    // if (process.env.NODE_ENV === 'production') { ... }. See `./env.js`.
    // It is absolutely essential that NODE_ENV is set to production
    // during a production build.
    // Otherwise React will be compiled in the very slow development mode.
    new webpack.DefinePlugin(env.stringified),
    // Experimental hot reloading for React .
    // https://github.com/facebook/react/tree/main/packages/react-refresh
    isEnvDevelopment && shouldUseReactRefresh && new ReactRefreshWebpackPlugin({
      overlay: false
    }),
    // Watcher doesn't work well if you mistype casing in a path so we use
    // a plugin that prints an error when you attempt to do this.
    // See https://github.com/facebook/create-react-app/issues/240
    isEnvDevelopment && new CaseSensitivePathsPlugin(), new MiniCssExtractPlugin({
      // Options similar to the same options in webpackOptions.output
      // both options are optional
      filename: 'static/css/[name].[contenthash:8].css',
      chunkFilename: 'static/css/[name].[contenthash:8].chunk.css'
    }),
    // Generate a manifest containing the required script / css for each entry.
    new WebpackManifestPlugin({
      fileName: 'entrypoint-manifest.json',
      publicPath: paths.publicUrlOrPath,
      generate: (seed, files, entrypoints) => {
        const entrypointFiles = (fileName => !('.map' |> fileName.endsWith(%))) |> entrypoints.main.filter(%);
        const processedEntrypoints = {};
        for (let key in entrypoints) {
          processedEntrypoints[key] = {
            js: (filename =>
            // Include JS assets but ignore hot updates because they're not
            // safe to include as async script tags.
            ('.js' |> filename.endsWith(%)) && !('.hot-update.js' |> filename.endsWith(%))) |> entrypoints[key].filter(%),
            css: (filename => '.css' |> filename.endsWith(%)) |> entrypoints[key].filter(%)
          };
        }
        return processedEntrypoints;
      }
    }),
    // Moment.js is an extremely popular library that bundles large locale files
    // by default due to how webpack interprets its code. This is a practical
    // solution that requires the user to opt into importing specific locales.
    // https://github.com/jmblog/how-to-optimize-momentjs-with-webpack
    // You can remove this if you don't use Moment.js:
    new webpack.IgnorePlugin({
      resourceRegExp: /^\.\/locale$/,
      contextRegExp: /moment$/
    }),
    // TypeScript type checking
    useTypeScript && new ForkTsCheckerWebpackPlugin({
      async: isEnvDevelopment,
      typescript: {
        typescriptPath: 'typescript' |> resolve.sync(%, {
          basedir: paths.appNodeModules
        }),
        configOverwrite: {
          compilerOptions: {
            sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
            skipLibCheck: true,
            inlineSourceMap: false,
            declarationMap: false,
            noEmit: true,
            incremental: true,
            tsBuildInfoFile: paths.appTsBuildInfoFile
          }
        },
        context: paths.appPath,
        diagnosticOptions: {
          syntactic: true
        },
        mode: 'write-references'
        // profile: true,
      },
      issue: {
        // This one is specifically to match during CI tests,
        // as micromatch doesn't match
        // '../cra-template-typescript/template/src/App.tsx'
        // otherwise.
        include: [{
          file: '../**/src/**/*.{ts,tsx}'
        }, {
          file: '**/src/**/*.{ts,tsx}'
        }],
        exclude: [{
          file: '**/src/**/__tests__/**'
        }, {
          file: '**/src/**/?(*.){spec|test}.*'
        }, {
          file: '**/src/setupProxy.*'
        }, {
          file: '**/src/setupTests.*'
        }]
      },
      logger: {
        infrastructure: 'silent'
      }
    }),
    // Fork Start
    new ReactFlightWebpackPlugin({
      isServer: false,
      clientReferences: {
        directory: './src',
        recursive: true,
        include: /\.(js|ts|jsx|tsx)$/
      }
    })
    // Fork End
    ].filter(%),
    experiments: {
      topLevelAwait: true
    },
    // Turn off performance processing because we utilize
    // our own hints via the FileSizeReporter
    performance: false
  };
};